// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

using System;
using System.Reflection;

namespace fit
{
	public class ColumnFixture : Fixture 
	{
		internal TypeAdapter[] columnBindings;

		// Traversal ////////////////////////////////

		public override void doRows(Parse rows) 
		{
			bind(rows.parts);
			base.doRows(rows.more);
		}

		public override void doCell(Parse cell, int column) 
		{
			TypeAdapter a = columnBindings[column];
			try 
			{
				if (a == null) 
				{
					ignore(cell);
				} 
				else if (a.field != null) 
				{
					String text = cell.text();
					if (text != "") 
					{
						a.set(a.parse(text));
					}
				} 
				else if (a.method != null) 
				{
					check(cell, a);
				}
			} 
			catch(Exception e) 
			{
				exception(cell, e);
			}
		}

		// Utility //////////////////////////////////

		internal void bind (Parse heads) 
		{
			columnBindings = new TypeAdapter[heads.size()];
			for (int i=0; heads!=null; i++, heads=heads.more) 
			{
				String name = heads.text();
				String suffix = "()";
				try 
				{
					if (name.EndsWith(suffix)) 
					{
						columnBindings[i] = bindMethod(name.Substring(0,name.Length-suffix.Length));
					} 
					else 
					{
						columnBindings[i] = bindField(name);
					}
				}
				catch (Exception e) 
				{
					exception (heads, e);
				}
			}

		}

		internal TypeAdapter bindMethod (string name) 
		{
			MethodInfo method = getTargetClass().GetMethod(name, new Type[] {});
			if (method == null) throw new MissingMethodException(getTargetClass().FullName, name);
			return TypeAdapter.on(this, method);
		}

		internal TypeAdapter bindField (string name) 
		{
			return TypeAdapter.on(this, getTargetClass().GetField(name));
		}

		protected internal virtual Type getTargetClass() 
		{
			return GetType();
		}
	}
}
